function [relerr,rmserr,abserr,absind]=errapprox(varargin)
%%
% ERRAPPROX                    Accuracy of an approximating function
% ------------------------------------------------------------------
%
% This routine computes several  indicators to evaluate the accuracy
% of a function y=f(x), which approximates a set  of real samples yk
% obtained for N different  values xk of the real-valued explanatory
% variables x.
%
% CALL
% [relerr,rmserr,abserr,absind]=errapprox(Y,fdata)
% [relerr,rmserr,abserr,absind]=errapprox(X,Y,names,flfr)
%
% INPUT ARGUMENTS
% - X: values xk of the explanatory variables x  (n x N array, where
%      X(:,k) corresponds to xk).
% - Y: real samples yk (n1 x n2 x N array where Y(:,:,k) corresponds
%      to yk in the general case, or possibly 1 x N array where Y(k)
%      corresponds to yk if n1=n2=1).
% - names: names of the variables (1 x n cell array of strings).
% - fdata: values f(xk)  of the approximating  function f (same size
%          as Y).
% - flfr: linear  fractional  representation  of  the  approximating
%         function f  (gss object if  the GSS Library is  installed,
%         lfr object otherwise if the LFR Toolbox is installed).
%
% OUTPUT ARGUMENTS
% - relerr: global relative error for each entry (n1 x n2 array).
% - rmserr: root-mean-square error for each entry (n1 x n2 array).
% - abserr: max local absolute error for each entry (n1 x n2 array).
% - absind: index of the  sample at which the maximum local absolute
%           error is computed for each entry (n1 x n2 array).
%
% SMAC TOOLBOX - APRICOT LIBRARY
% See http://w3.onera.fr/smac/?q=errapprox for more info!

% ------------------------------------------------------------------
% SMAC Toolbox - ONERA The French Aerospace Lab, Toulouse, France
% Author: Clément Roos
% Last revision: March 2016 by Clément Roos
% ------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Preliminary tests and initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if nargin~=2 && nargin~=4
  error('ERRAPPROX: Wrong number of input arguments.');
end
if nargin==2
  data=varargin{1};
  fdata=varargin{2};
else
  par.values=varargin{1};
  data=varargin{2};
  par.names=varargin{3};
  flfr=varargin{4};
end
s=size(data);
if length(s)==2
  data=reshape(data,s(1),1,s(2));
elseif length(s)~=3
  error('ERRAPPROX: The variable Y must be a 2-D or a 3-D array.');
end
if nargin==2
  s=size(fdata);
  if length(s)==2
    fdata=reshape(fdata,s(1),1,s(2));
  end
  if size(data,1)~=size(fdata,1) || size(data,2)~=size(fdata,2)
    error('ERRAPPROX: The variables Y and fdata are not compatible.');
  end
else
  if size(par.values,1)~=length(par.names)
    error('ERRAPPROX: The variables X and names are not compatible.');
  end
  if size(data,3)~=size(par.values,2)
    error('ERRAPPROX: The variables X and Y are not compatible.');
  end
  if isa(flfr,'gss')
    if ~exist('gss.m','file')
      error('ERRAPPROX: The GSS Library is not installed.')
    end
    isgss=1;
  elseif isa(flfr,'lfr')
    if ~exist('lfr.m','file')
      error('ERRAPPROX: The LFR Toolbox is not installed.')
    end
    isgss=0;
  else
    error('ERRAPPROX: The variable flfr must be a gss or a lfr object.');
  end
  if size(data,1)~=size(flfr,1) || size(data,2)~=size(flfr,2)
    error('ERRAPPROX: The variables Y and flfr are not compatible.');
  end
  if isgss
    for k=1:length(flfr.D)
      if all(strcmp(flfr.D(k).Name,par.names)==0)
        error('ERRAPPROX: The variables names and flfr are not compatible.');
      end
    end
  else
    for k=1:length(flfr.blk.names)
      if all(strcmp(flfr.blk.names{k},par.names)==0)
        error('ERRAPPROX: The variables names and flfr are not compatible.');
      end
    end
  end
end
n=size(data);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluation of the LFR object on the grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if nargin==4
  fdata=zeros(size(data));
  if isgss
    flfr0=eval(flfr,par.names,num2cell(par.values'));
    for k=1:n(3)
      fdata(:,:,k)=flfr0{k};
    end
  else
    for k=1:n(3)
      flfr0=uplft(flfr,par.names,par.values(:,k)');
      fdata(:,:,k)=flfr0.d;
    end
  end
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computation of the approximation errors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
err=data-fdata;
zn=zeros(n(1:2));
relerr=zn;
rmserr=zn;
abserr=zn;
absind=zn;
for k1=1:n(1)
  for k2=1:n(2)
    relerr(k1,k2)=norm(squeeze(err(k1,k2,:)))/norm(squeeze(data(k1,k2,:)));
    rmserr(k1,k2)=norm(squeeze(err(k1,k2,:)))/sqrt(n(3));
    abserr(k1,k2)=max(abs(squeeze(err(k1,k2,:))));
    absind(k1,k2)=find(abs(squeeze(err(k1,k2,:)))==abserr(k1,k2),1);
  end
end
