%%
% KOALA              Rational approximation using Surrogate Modeling
% ------------------------------------------------------------------
%
% This routine uses Surrogate Modeling to compute a multivariate ra-
% tional approximation y=f(x) of  a set of q x 1 real vector samples
% yk obtained for  N different values xk of the real-valued explana-
% tory variables x. The  maximum degree of the rational  function is
% set by  the user and a factorized  expression is obtained.  Unless
% specified, its nonsingularity  is guaranteed on the whole conside-
% red domain (defined as the convex hull of the xk).
%
% CALL
% [fdata,fdesc,fsym,flfr,Vloo,CI]=koala(X,Y,names,{maxdeg,options})
%
% INPUT ARGUMENTS
% - X: values xk of the explanatory variables x  (n x N array, where
%      X(:,k) corresponds to xk).
% - Y: real samples yk to be approximated  (q x N array where Y(:,k)
%      corresponds to yk).
% - names: names of the variables (1 x n cell array of strings).
% - maxdeg: maximum degree of the approximating function (even inte-
%           ger, default=100).  maxdeg/2 kernels are  created unless
%           options.target is defined and reached with less kernels.
% The fifth input argument  is an optional structured  variable. De-
% fault values  are given between  brackets. Most  of the parameters
% can be left to their default setting, some of them (swarm topology
% PSO parameters) being especially devoted to experienced users. The
% most important ones are target and rbf.
% ---General and modeling issues---
%   * weights: [ones(1,N)] vector  containing the  weighting coeffi-
%              cients applied to the data samples. if q>1, it should
%              remain a  vector,  and the  weights  are  identically
%              applied to the data samples for each output.
%   * target: [0] minimum criterion value for stopping  (choose 0 to
%             run till degree maxdeg (if defined) or till 50 kernels
%             at most).
%   * rbf: [0] type of modeling involved in the local approximation:
%               * 0 = LLM (Local Linear Model)
%               * 1 = standard RBF network
%   * type: [1] type of radial cell:
%               * 0 for Gaussian [e^(-r^2)]
%               * 1 for Pade order 1 [1/(1+r^2)]
%               * 2 for pseudo-Pade [1/(1+|r|)]
%               note: 0 and 2 are not available for LFR modeling.
%   * B: [] n x 4 x nz array containing the lower/upper bounds asso-
%        ciated to the kernel parameters,  where nz is the number of
%        data zones  (see options.nbzone). The different  bounds are
%        splitted along the 2nd dimension as follows:
%               * B(:,1,:) for the lower bounds of the centers
%               * B(:,2,:) for the lower bounds of the radii
%               * B(:,3,:) for the upper bounds of the centers
%               * B(:,4,:) for the upper bounds of the radii
%   * snls: [5] number  of iterations  between  Separable  Nonlinear
%           Least Squares (SNLS) optimizations (Inf if no SNLS opti-
%           mization, negative number to speed up the SNLS stages).
%   * loo: [1] optimization based on a Leave-One-Out/Predicted Resi-
%          dual Sum of Squares (LOO/PRESS)  generalization criterion
%          (default) or on the usual sum of squares.
%   * trace: trace of execution (0=no, 1=text, 2=text+figs, def=1).
%   * viewpoint: modification  of the  view from  which an  observer
%                sees 3-D plots. viewpoint(1) is the  horizontal ro-
%                tation and  viewpoint(2) is the vertical  elevation
%                in degrees (def=[0 0]).
% ---Convergence parameters---
%   * maxit: [200] maximum number of PSO iterations.
%   * cvgce1: [0.05] relative variation of criterion (%).
%   * cvgce2: [5] normalized average speed of swarm particles (%).
%   * nbstep2stop: [4] number of successive steps required to satis-
%                  fy the stopping criteria.
% ---Topology of the swarms---
%   * nbswarm: [1] number of swarms competing for the best result or
%              acting in several sub-domains.
%   * nbzone: [1] number of optimization sub-domains (nz).
%   * nbtrial: [1] number of  PSO strategies / topologies to  be run
%              successively in case of failure.
%   * nbpart: [16] number of particles in each swarm.
%   * charged: [0] creating charged particles in each swarm in addi-
%              tion to the usual neutral ones.
% ---Processing the best result of each PSO iteration---
%   * mutation: [0] mutation process  of the (global)  best particle
%               (especially useful for multimodal problems).
%   * stdmax: [0.5] stantard  max bound for  the normal distribution
%             of the mutation process.
%   * stdmin: [0.5] standard  min bound for  the normal distribution
%             of the mutation process.
%   * local: [1] local  optimization  to improve  the (global)  best
%            particle.
%   * delaunay: [0] Delaunay tessellation  for computing best neigh-
%               bors (dynamic topologies).
%   * neldermead: [0] local optimization using Nelder-Mead algorithm
%                 in addition.
% ---Cross-validation and confidence intervals---
%   * Xv: [] n x M  array containing  the values of  the explanatory
%         variables used for cross-validation (M samples).
%   * Yv: [] q x M array containing the samples used for cross-vali-
%         dation (M samples).
%   * Xci: [] n x P array  containing the values  of the explanatory
%          variables  used to  compute the  confidence  intervals (P
%          samples).
%
% OUTPUT ARGUMENTS
% - fdata: values f(xk)  of the approximating  function f (same size
%          as Y).
% - fdesc: structured variable describing the approximating function
%          * nbker: number of kernels (radial units) involved in the
%            final model.
%          * centers: n x nbker array  containing the  components of
%            the nbker kernel centers.
%          * radii: n x nbker array containing the components of the
%            nbker kernel radii.
%          * teta: nbker x nbw array containing the regression para-
%            meters in the user model space, with nbw=1 for standard
%            RBF networks (options.rbf=1)  or nbw=n+1 for Local  Li-
%            near Models (options.rbf=0).
% - fsym: associated symbolic representation (symbolic object).
% - flfr: associated linear fractional representation (gss object if
%         the GSS Library is installed,  lfr object otherwise if the
%         LFR Toolbox is installed). even if this cannot  be proved,
%         its size is usually equal to n*maxdeg.
% - Vloo: variance of the outputs  (Leave-One-Out type)  computed on
%         the estimation data X (q x N array).
% - CI: Confidence Interval (99%) of the outputs computed on the da-
%       ta Xci (q x P array).
%
% CAUTION
% If some values in X are much larger than 1, they must be scaled or
% normalized.
%
% SMAC TOOLBOX - APRICOT LIBRARY
% See http://w3.onera.fr/smac/?q=koala for more info!
