%%
% OLSAPPROX  Polynomial approximation using Orthogonal Least-Squares
% ------------------------------------------------------------------
%
% This routine uses  Orthogonal Least-Squares  to compute a multiva-
% riate  polynomial approximation  y=f(x)  of a set of  n1 x n2 real
% samples yk obtained  for N different values xk  of the real-valued
% explanatory variables x. The  maximum degree of the polynomial and
% the maximum admissible approximation error  are set by the user. A
% sparse  polynomial  expression is usually obtained, which  ensures
% that the global relative error or  the root-mean-square error bet-
% ween f(xk) and yk remains below the given threshold for each entry
%
% CALL
% [fdata,fdesc,fsym,flfr]=olsapprox(X,Y,names,maxdeg,maxerr,options)
%
% INPUT ARGUMENTS
% - X: values xk of the explanatory variables x  (n x N array, where
%      X(:,k) corresponds to xk).
% - Y: real samples yk  to be approximated  (n1 x n2 x N array where
%      Y(:,:,k)  corresponds to yk  in the general case, or possibly
%      1 x N array where Y(k) corresponds to yk if n1=n2=1).
% - names: names of the variables (1 x n cell array of strings).
% - maxdeg: maximum degree of the approximating function.
% - maxerr: maximum admissible approximation error (if possible).
%           . maxerr>=0: the  global relative  error  norm(Y-fdata)/
%             norm(Y) remains below maxerr for each entry.
%           . maxerr<0: the  root-mean-square  error  norm(Y-fdata)/
%             sqrt(N) remains below abs(maxerr) for each entry.
%           . maxerr=[]: default tolerances  are used and  the error
%             cannot be mastered, but the algorithm is faster.
% - options is an optional structured variable with fields:
%   * maxexp: maximum exponent of each variable in the approximating
%             function (1 x n array, def=maxdeg*ones(1,n)).
%   * abserr: maximum admissible local absolute error abs(Y(i1,i2,k)
%             - fdata(i1,i2,k)) over all samples 1<=k<=N and all en-
%             tries 1<=i1<=n1 and 1<=i2<=n2 (if possible) (def=Inf).
%   * pendeg: penalization of  each monomial  proportionally  to its
%             degree (0 = no penalty, 1 = medium penalty, >1 = large
%             penalty, def=1).
%   * trace: trace of execution (0=no, 1=text, 2=text+figs, def=1).
%   * viewpoint: deviation  with respect  to the  default  viewpoint
%                (see plotapprox, def=[0 0]). applicable only if 3-D
%                graphs are to be displayed (options.trace=2 & n=2).
%   * warn: warnings display (0=no, 1=yes, def=1).
%
% OUTPUT ARGUMENTS
% - fdata: values f(xk)  of the approximating  function f (same size
%          as Y).
% - fdesc: description of the approximating function  (fdesc{i1,i2}.
%          coef(j) and fdesc{i1,i2}.exp(j,:) contain the coefficient
%          and the exponents of the jth monomial used to approximate
%          entry (i1×i2) of Y). 
% - fsym: associated symbolic representation (symbolic object).
% - flfr: associated linear fractional representation (gss object if
%         the GSS Library is installed,  lfr object otherwise if the
%         LFR Toolbox is installed).
%
% NOTE
% A structure can be defined for the appoximating polynomial. Assume
% e.g. that an expression of the form x1*x3^2*g(x1,x2)+x2^3*h(x1,x4)
% is searched, which depends on 4 variables  x1,...,x4 and is compo-
% sed of two polynomials g and h. Three additional fields are intro-
% duced in the input variable options:
% - options.parin={[1 2] [1 4]}
% - options.parout={[1 3] [2]}
% - options.expout={[1 2] [3]}
% By default, no structure is considered, so options.parin={1:npar},
% options.parout={[]} and options.expout={[]}.
%
% CAUTION
% If some values in X are much larger than 1, they must be scaled or
% normalized.
%
% SMAC TOOLBOX - APRICOT LIBRARY
% See http://w3.onera.fr/smac/?q=olsapprox for more info!
