function plotapprox(X,Y,names,fdata,options,info)
%%
% PLOTAPPROX   Graphical representation of an approximating function
% ------------------------------------------------------------------
%
% This routine displays a graphical comparison between a set of real
% samples yk obtained  for N different values xk of some real-valued
% explanatory variables x, and  an associated approximating function
% y=f(x). The local approximation error f(xk)-yk is plotted for each
% entry and several indicators are computed to evaluate the accuracy
% of the approximation (see errapprox).
%
% CALL
% plotapprox(X,Y,names,fdata,options)
% plotapprox(X,Y,names,flfr,options)
%
% INPUT ARGUMENTS
% - X: values xk of the explanatory variables x  (n x N array, where
%      X(:,k) corresponds to xk). note that n can only be equal to 1
%      or 2 in this routine.
% - Y: real samples yk (n1 x n2 x N array where Y(:,:,k) corresponds
%      to yk in the general case, or possibly 1 x N array where Y(k)
%      corresponds to yk if n1=n2=1).
% - names: names of the variables (1 x n cell array of strings).
% - fdata: values f(xk)  of the approximating  function f (same size
%          as Y).
% - flfr: linear  fractional  representation  of  the  approximating
%         function f  (gss object if  the GSS Library is  installed,
%         lfr object otherwise if the LFR Toolbox is installed).
% - options is an optional structured variable with field:
%   * viewpoint: modification  of the  view from  which an  observer
%                sees 3-D plots. viewpoint(1) is the  horizontal ro-
%                tation and  viewpoint(2) is the vertical  elevation
%                in degrees (def=[0 0]).
%
% SMAC TOOLBOX - APRICOT LIBRARY
% See http://w3.onera.fr/smac/?q=plotapprox for more info!

% ------------------------------------------------------------------
% SMAC Toolbox - ONERA The French Aerospace Lab, Toulouse, France
% Author: Clément Roos
% Last revision: January 2022 by Clément Roos
% ------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Preliminary tests and initialization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if nargin<4 || nargin>6
  error('PLOTAPPROX: Wrong number of input arguments.');
end
data=Y;
par.values=X;
par.names=names;
s=size(data);
if length(s)==2
  data=reshape(data,s(1),1,s(2));
elseif length(s)~=3
  error('PLOTAPPROX: The variable Y must be a 2-D or a 3-D array.');
end
if length(par.names)>2
  error('PLOTAPPROX: Only problems with one or two parameters can be considered');
end
if size(par.values,1)~=length(par.names)
  error('PLOTAPPROX: The variables X and names are not compatible.');
end
if size(data,3)~=size(par.values,2)
  error('PLOTAPPROX: The variables X and Y are not compatible.');
end
cdata=class(fdata);
switch cdata
  case 'gss'
    if ~exist('gss.m','file')
      error('PLOTAPPROX: The GSS Library is not installed.')
    end
    isgss=1;
    syslfr=fdata;
    if size(data,1)~=size(syslfr,1) || size(data,2)~=size(syslfr,2)
      error('PLOTAPPROX: The variables Y and flfr are not compatible.');
    end
    for k=1:length(syslfr.D)
      if all(strcmp(syslfr.D(k).Name,par.names)==0)
        error('ERRAPPROX: The variables names and flfr are not compatible.');
      end
    end
  case 'lfr'
    if ~exist('lfr.m','file')
      error('PLOTAPPROX: The LFR Toolbox is not installed.')
    end
    isgss=0;
    syslfr=fdata;
    if size(data,1)~=size(syslfr,1) || size(data,2)~=size(syslfr,2)
      error('PLOTAPPROX: The variables Y and flfr are not compatible.');
    end
    for k=1:length(syslfr.blk.names)
      if all(strcmp(syslfr.blk.names{k},par.names)==0)
        error('PLOTAPPROX: The variables names and flfr are not compatible.');
      end
    end
  otherwise
    s=size(fdata);
    if length(s)==2
      fdata=reshape(fdata,s(1),1,s(2));
    end
    if size(data,1)~=size(fdata,1) || size(data,2)~=size(fdata,2)
      error('PLOTAPPROX: The variables Y and fdata are not compatible.');
    end
end
if nargin==4
  options=[];
  info=[];
elseif nargin==5
  info=[];
end
if isfield(options,'viewpoint')
  viewpoint=options.viewpoint;
  if numel(viewpoint)~=2
    error('PLOTAPPROX: The variable options.viewpoint must be a 1x2 vector');
  end
else
  viewpoint=[0 0];
end
n=size(data);
npar=size(par.values,1);
if ~isfield(info,'npts')
  info.npts=n(3);
end
if ~isfield(info,'size')
  info.size=[n(1) n(2)];
end
if ~isfield(info,'npar')
  info.npar=npar;
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Evaluation of the LFR object on the grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if any(strcmp(cdata,{'gss' 'lfr'}))
  fdata=zeros(size(data));
  if isgss
    syslfr0=eval(syslfr,par.names,num2cell(par.values'));
    for k=1:n(3)
      fdata(:,:,k)=syslfr0{k};
    end
    if ~isfield(info,'sizelfr')
      info.sizelfr=size(syslfr,'blk');
    end    
  else
    for k=1:n(3)
      syslfr0=uplft(syslfr,par.names,par.values(:,k)');
      fdata(:,:,k)=syslfr0.d;
    end
    if ~isfield(info,'sizelfr')
      info.sizelfr=sum(syslfr.blk.desc(1,:));
    end
  end
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Computation of the approximation errors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
if ~isfield(info,'relerr')
  [info.relerr,info.rmserr,info.abserr,info.absind]=errapprox(data,fdata);
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Results display
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
for k1=1:n(1)
  for k2=1:n(2)
    figure,fullscreen
    set(gcf,'color','w')
    if npar==1
      xmin=min(par.values);
      xmax=max(par.values);
      ymin=min(squeeze(data(k1,k2,:)));
      ymax=max(squeeze(data(k1,k2,:)));
      subplot(2,2,1)
      plot(par.values,squeeze(data(k1,k2,:)));
      hold on,grid on
      plot(par.values,squeeze(data(k1,k2,:)),'b.');
      xlim([xmin-eps xmax+eps]);
      ylim([ymin-(ymax-ymin)/10 ymax+(ymax-ymin)/10]);
      xlabel(par.names{1},'FontSize',12);
      title('Reference data','FontSize',12);
      subplot(2,2,2)
      plot(par.values,squeeze(fdata(k1,k2,:)));
      grid on
      xlim([xmin-eps xmax+eps]);
      ylim([ymin-(ymax-ymin)/10 ymax+(ymax-ymin)/10]);
      xlabel(par.names{1},'FontSize',12);
      title('Approximated data','FontSize',12);
      subplot(2,2,3)
      plot(par.values,squeeze(fdata(k1,k2,:)-data(k1,k2,:)));
      hold on,grid on
      plot(par.values,squeeze(fdata(k1,k2,:)-data(k1,k2,:)),'b.');
      xlim([xmin-eps xmax+eps]);
      xlabel(par.names{1},'FontSize',12);
      title('Local approximation error','FontSize',12);
    elseif npar==2
      xmin=min(par.values(1,:));
      xmax=max(par.values(1,:));
      ymin=min(par.values(2,:));
      ymax=max(par.values(2,:));
      zmin=min(squeeze(data(k1,k2,:)));
      zmax=max(squeeze(data(k1,k2,:)));
      subplot(2,2,1)
      tri=delaunay(par.values(1,:),par.values(2,:));
      trisurf(tri,par.values(1,:),par.values(2,:),squeeze(data(k1,k2,:)));
      % h=trimesh(tri,par.values(1,:),par.values(2,:),squeeze(data(k1,k2,:)));
      % set(h,'EdgeAlpha',0,'Marker','.','MarkerFaceColor','flat','MarkerSize',6);
      view([-37.5 30]+viewpoint);
      xlim([xmin-eps xmax+eps]);
      xlabel(par.names{1},'FontSize',12);
      ylim([ymin-eps ymax+eps]);
      ylabel(par.names{2},'FontSize',12);
      zlim([zmin-(zmax-zmin)/10 zmax+(zmax-zmin)/10]);
      title('Reference data','FontSize',12);
      subplot(2,2,2)
      trisurf(tri,par.values(1,:),par.values(2,:),squeeze(fdata(k1,k2,:)));
      view([-37.5 30]+viewpoint);
      xlim([xmin-eps xmax+eps]);
      xlabel(par.names{1},'FontSize',12);
      ylim([ymin-eps ymax+eps]);
      ylabel(par.names{2},'FontSize',12);
      zlim([zmin-(zmax-zmin)/10 zmax+(zmax-zmin)/10]);
      title('Approximated data','FontSize',12);
      subplot(2,2,3)
      trisurf(tri,par.values(1,:),par.values(2,:),squeeze(fdata(k1,k2,:)-data(k1,k2,:)));
      view([-37.5 30]+viewpoint);
      xlim([xmin-eps xmax+eps]);
      xlabel(par.names{1},'FontSize',12);
      ylim([ymin-eps ymax+eps]);
      ylabel(par.names{2},'FontSize',12);
      title('Local approximation error','FontSize',12);
    end
    subplot(2,2,4),axis off,hold on
    text('Interpreter','none','String',{'Systems Modeling Analysis and Control (SMAC) Toolbox';...
         'APRICOT Library';'Version 1.2 - January 2022'},...
         'Position',[-0.06 .95],'FontUnits','normalized','FontSize',0.062,'FontWeight','bold');
    ypos=0.72;
    if isfield(info,'method')
      method='                      ';
      method(1:length(info.method))=info.method;
      text('Interpreter','none','String',['Polynomial approximation using ' method],...
           'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
      ypos=ypos-0.13;
    end
    text('Interpreter','none','String',['Number of samples: ' num2str(info.npts)],...
         'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
    ypos=ypos-0.07;
    text('Interpreter','none','String',['Size of samples: ' num2str(info.size(1)) 'x' num2str(info.size(2))],...
         'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
    ypos=ypos-0.07;
    if n(1)*n(2)>1
      text('Interpreter','none','String',['Considered entry: (' num2str(k1) 'x' num2str(k2) ')'],...
           'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
      ypos=ypos-0.07;
    end
    names=par.names{1};
    if npar==2
      names=[names ' and ' par.names{2}];
    end
    text('Interpreter','none','String',['Number of explanatory variables: ' num2str(info.npar) ' (' names ')'],...
         'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
    ypos=ypos-0.07;
    if isfield(info,'maxdeg')
      text('Interpreter','none','String',['Maximum degree of the approximating function: ' num2str(info.maxdeg)],...
           'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
      ypos=ypos-0.07;
    end
    if isfield(info,'maxexp')
      maxexp='';
      for k=1:length(info.maxexp)
        maxexp=[maxexp num2str(info.maxexp(k)) ' '];
      end
      text('Interpreter','none','String',['Maximum degree of each explanatory variable: ' maxexp],...
           'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
      ypos=ypos-0.07;
    end
    ypos=ypos-0.06;
    if isfield(info,'sizelfr')
      text('Interpreter','none','String',['Size of the LFR: ' num2str(info.sizelfr)],...
           'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
      ypos=ypos-0.13;
    end
    text('Interpreter','none','String',['Global relative error: ' num2str(info.relerr(k1,k2),'%8.3e')],...
         'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
    ypos=ypos-0.07;
    text('Interpreter','none','String',['Root-mean-square error: ' num2str(info.rmserr(k1,k2),'%8.3e')],...
         'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
    ypos=ypos-0.07;
    text('Interpreter','none','String',['Maximum local absolute error: ' num2str(info.abserr(k1,k2),'%8.3e') ' at point ' num2str(info.absind(k1,k2))],...
         'Position',[-0.06 ypos],'FontUnits','normalized','FontSize',0.059,'FontWeight','bold');
    test=imread('smac','jpg');
    axes('Position',[0.9 0 0.1 0.1],'Xtick',[],'Ytick',[],'box','on');
    imagesc(test);
    axis off
    axis image
  end
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% <<< SUBFUNCTION FULLSCREEN >>>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function fullscreen(h)
%
% FULLSCREEN               Set a figure to almost full-screen mode
% ----------------------------------------------------------------
%
% This function sets a figure to almost full-screen mode.
%
% CALL: fullscreen({h})
%
% INPUTS
%  - h: figure handle. it unspecified, the modification applies to
%       the current figure.

% ----------------------------------------------------------------
% Author: Clément Roos (ONERA)
% Date: August 2010
% ----------------------------------------------------------------
%
if nargin==0
  h=gcf;
end
s=get(0,'ScreenSize');
set(h,'Position',[50 80 s(3)-100 s(4)-160-50]);
