%%
% TRACKER           Rational approximation using Genetic Programming
% ------------------------------------------------------------------
%
% This routine  uses Genetic  Programming to compute  a multivariate
% rational approximation  y=f(x) of a set of real  scalar samples yk
% obtained for N different  values xk of the real-valued explanatory
% variables x. The maximum degree of the rational function is set by
% the user and a sparse rational expression is usually obtained. Un-
% less specified, its nonsingularity is guaranteed on the whole con-
% sidered  domain (defined as the convex hull of the xk).
%
% CALL
% [pop,best,fdata,fdesc,fsym,flfr]=tracker(X,Y,names,maxdeg,options)
% 
% INPUT ARGUMENTS
% - X: values xk of the explanatory variables x  (n x N array, where
%      X(:,k) corresponds to xk).
% - Y: real samples  yk to be  approximated  (1 x N array where Y(k)
%      corresponds to yk).
% - names: names of the variables (1 x n cell array of strings).
% - maxdeg: maximum degree of the approximating function.
% The fifth input argument  is an optional structured  variable. De-
% fault values  are given between  brackets. Most  of the parameters
% can be left to their default setting, some of them (tree features,
% evolutionary  process)  being  especially  devoted  to experienced
% users. The most important ones are maxexp, nbind and nbgen.
% ---User constraints and settings---
%   * maxexp: [maxdeg*ones(1,n)] maximum  exponent of  each variable
%             in the approximating function.
%   * weights: [ones(1,N)] vector  containing the  weighting coeffi-
%              cients applied to the data samples.
%   * trace: trace of execution (0=no, 1=text, 2=text+figs, def=1).
%   * viewpoint: modification  of the  view from  which an  observer
%                sees 3-D plots. viewpoint(1) is the  horizontal ro-
%                tation and  viewpoint(2) is the vertical  elevation
%                in degrees (def=[0 0]).
% ---Population features---
%   * nbind: [100] number of individuals in the population.
%   * nbgen: [100] maximum number of generations.
%   * display: [5] display rate in number of generations.
% ---Convergence parameters---
%   * goal: [false] to stop if a minimum fitness value is reached.
%   * target: [0] minimum fitness value for stopping.
%     Remark: Stopping the run at any time is also possible by chan-
%             ging a  flag from 0 to 1 in the file  "stop.dat" (flag
%             checked by the code at the end of every generation).
% ---Set of terminal nodes---
%   * nbfvar: [-1] number of  fictitious variables added to get sim-
%             pler trees (nbfvar<0  to let the  fictitious variables
%             be automatically determined w.r.t. maxexp, nbfvar=0 to
%             avoid any fictitious  variable, nbfvar>0 to choose the
%             number and type of fictitious variables).
%   * fvar: [[]] vector containing the numbers of explanatory varia-
%           bles associated  to the fictitious variables (to be pro-
%           vided only if nbfvar>0).
%   * dfvar: [[]] vector containing  the relative degrees of the ex-
%            planatory variables associated to the fictitious varia-
%            bles (to be provided only if nbfvar>0).
% ---Tree features---
%   * treedepth: [13] maximum allowed depth of trees (can be automa-
%                tically  incremented during  the run till the tree-
%                depthigh value is reached).
%   * treenodes: [200] maximum allowed number of nodes per tree.
%   * treealgo: [3] method used to generate the trees (1 = full, 2 =
%               grow, 3 = ramp half and half).
%   * subtreedepth: [min(8,fix(2*treedepth/3))] max allowed depth of
%                   sub-trees when using mutation operator.
%   * treedepthigh: [18] max maximorum value of the treedepth param.
% ---Evolutionary process---
%   * pmutation: [0.10] probability of mutation.
%   * pcrossover: [0.85] probability of crossover.
%   * pcopy: [0.05] probability of direct reproduction.
%     Remark: pmutation + pcrossover + pcopy must be 1.
%   * pmutation_subtree: [0.90] probability of sub-tree  mutation.
%   * pmutation_termswitch: [0.10] probability  of exchange  between
%                           terminal nodes.
%     Remark: pmutation_subtree + pmutation_termswitch must be 1.
%   * toursize: [4] tournament size during the selection process.
%   * highlevcross: [0.5] proportion of high level crossover allowed
%                   for '+'-type nodes.
%   * parsipress: [false] lexicographic parsimony pressure.
%   * elitism: [0.05] elite  proportion of  the population  kept for
%              the following generation.
%
% OUTPUT ARGUMENTS
% - pop: structure containing the main information about the indivi-
%        duals of the final population.
%   * nbindiv: number of individuals in the population.
%   * nbinputs: number of explanatory variables including fictitious
%               variables if any.
%   * nbfinputs: number of fictitious variables added to get simpler
%                trees.
%   * f2nfinputs: 1 x nbfinputs row vector containing the numbers of
%                 explanatory variables associated to the fictitious
%                 variables (only if nbfvar~=0).
%   * degfinputs: 1 x nbfinputs row  vector containing the  relative
%                 degrees of the explanatory variables associated to
%                 the fictitious variables (only if nbfvar~=0).
%   * indiv: cell vector  containing a  representation of  the parse
%            trees associated to  the individuals  of the population
%            (each component includes 2 cells of char containing the
%            chromosomes associated to the numerator pop.indiv{i}(1)
%            and denominator pop.indiv{i}(2)).
%   * fitness: vector containing the fitnesses of the individuals of
%              the population.
%   * nbnodes: vector containing the numbers of nodes of the indivi-
%              duals of the population.
%   * param: cell vector containing the monomials coefficients (vec-
%            tor) for the  individuals of the population  ordered as
%            follows:
%            - param{i}(1) = constant term of the numerator for the
%              ith individual of the population.
%            - param{i}(2:l+1) = coefficients  associated  to  the l
%              numerator monomials for the ith individual of the po-
%              pulation (apart from the constant term).
%            - param{i}(l+2:l+m+1) = coefficients  associated to the
%              m denominator monomials for the ith individual of the
%              population (apart from the constant term frozen to 1)
%   * nbreg: vector containing  the number of  regressors associated
%            to any individual in the population (l+m).
%   * degmax: vector containing the max degree of the regressors as-
%             sociated to any individual in the population.
%   * sumdeg: vector containing the  total degree  associated to any
%             individual in the population.
% - best: index of the best individual in the final population (with
%         respect to fitness).
% - fdata: values f(xk)  of the approximating  function f correspon-
%          ding to the best individual (same size as Y).
% - fdesc: description of  the approximating function  corresponding
%          to the best individual  (fdesc.cnum(j)/cden(j) and fdesc.
%          enum(j,:)/eden(j,:) contain the coefficient and the expo-
%          nents of the  jth monomial  of the  numerator/denominator
%          used to approximate Y).
% - fsym: associated symbolic representation (symbolic object).
% - flfr: associated linear fractional representation (gss object if
%         the GSS Library is installed,  lfr object otherwise if the
%         LFR Toolbox is installed).
%
% CAUTION
% If some values in X are much larger than 1, they must be scaled or
% normalized.
%
% SMAC TOOLBOX - APRICOT LIBRARY
% See http://w3.onera.fr/smac/?q=tracker for more info!
